@@ -2,7 +2,7 @@
 
 ;;; File: "_host.scm"
 
-;;; Copyright (c) 1994-2022 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2025 by Marc Feeley, All Rights Reserved.
 
 ;;;============================================================================
 
@@ -193,6 +193,14 @@
   (write obj port)
   1)
 
+;; 'write-returning-len-without-read-macros' is like 'write' but it
+;; returns the number of characters that were written out and it
+;; does not use read-macros for quote, quasiquote, etc.
+
+(define (write-returning-len-without-read-macros obj port)
+  (write obj port)
+  1)
+
 ;; 'display-returning-len' is like 'display' but it returns the number
 ;; of characters that were written out.
 
@@ -256,6 +264,13 @@
 (define (format-filepos path filepos pinpoint?)
   #f)
 
+(define (**display-message-with-locat gen-message locat kind proc port)
+  (display "*** " port)
+  (display kind port)
+  (locat-show " IN " locat port)
+  (display " -- " port)
+  (gen-message port))
+
 ;; The following functions define an interface to the file system's
 ;; naming conventions.  The current implementation is suitable for UNIX.
 ;;
@@ -727,7 +742,12 @@
   (##structure? obj))
 
 (define (structure->list obj)
-  (##vector->list obj))
+  (let ((len (##structure-length obj)))
+    (let loop ((i (- len 1)) (lst '()))
+      (if (< i 0)
+          lst
+          (loop (- i 1)
+                (cons (##unchecked-structure-ref obj i #f #f) lst))))))
 
 (define (open-input-file* path)
   (##open-file-generic
@@ -764,6 +784,33 @@
     (display str port)
     (string-length str)))
 
+(define (write-returning-len-without-read-macros obj port)
+
+  (define (readtable-without-read-macros)
+    (include "~~lib/_gambit#.scm")
+    (let ((rt (##make-standard-readtable)))
+      (macro-readtable-quote-keyword-set! rt #f)
+      (macro-readtable-quasiquote-keyword-set! rt #f)
+      (macro-readtable-unquote-keyword-set! rt #f)
+      (macro-readtable-unquote-splicing-keyword-set! rt #f)
+      (macro-readtable-sharp-quote-keyword-set! rt #f)
+      (macro-readtable-sharp-quasiquote-keyword-set! rt #f)
+      (macro-readtable-sharp-unquote-keyword-set! rt #f)
+      (macro-readtable-sharp-unquote-splicing-keyword-set! rt #f)
+      (macro-readtable-sharp-num-keyword-set! rt #f)
+      (macro-readtable-sharp-seq-keyword-set! rt #f)
+      rt))
+
+  (##namespace ("" with-output-to-string))
+
+  (let ((str
+         (with-output-to-string
+           (list readtable: (readtable-without-read-macros))
+           (lambda ()
+             (write obj)))))
+    (display str port)
+    (string-length str)))
+
 (define (display-returning-len obj port)
   (##namespace ("" with-output-to-string))
   (let ((str (with-output-to-string '() (lambda () (display obj)))))
@@ -782,7 +829,9 @@
 
 (define (in-char-range? n)
   (##declare (generic)) ; in case n is a bignum
-  (<= n ##max-char))
+  (and (not (< #x10ffff n)) ;; TODO: after bootstrap replace with (##max-char-code)
+        (or (< n #xd800)
+            (< #xdfff n))))
 
 (define (in-integer-range? n lo hi)
   (##declare (generic)) ; in case n is a bignum
@@ -806,6 +855,40 @@
 (define (format-filepos path filepos pinpoint?)
   (##format-filepos path filepos pinpoint?))
 
+;; TODO: remove after bootstrap
+(define (**make-locat container start-position end-position)
+  (if end-position
+      (vector container start-position end-position)
+      (vector container start-position)))
+
+;; TODO: remove after bootstrap
+(define (**locat-start-position locat)
+  (let ((container (vector-ref locat 0)))
+    (if (##source? container)
+        (**locat-start-position (source-locat container))
+        (vector-ref locat 1))))
+
+;; TODO: remove after bootstrap
+(define (**locat-end-position locat)
+  (let ((container (vector-ref locat 0)))
+    (if (##source? container)
+        (**locat-end-position (source-locat container))
+        (and (= (vector-length locat) 3)
+             (vector-ref locat 2)))))
+
+(define (**display-message-with-locat gen-message locat kind proc port)
+  (let ((dmwl
+         (##global-var-ref
+          (##make-global-var '##display-message-with-locat))))
+    (if (##unbound? dmwl) ;; TODO: remove after bootstrap
+        (begin
+          (display "*** " port)
+          (display kind port)
+          (locat-show " IN " locat port)
+          (display " -- " port)
+          (gen-message port))
+        (dmwl gen-message locat kind proc port))))
+
 ;; The path functions are already defined by Gambit
 ;;(define path-expand path-expand)
 ;;(define path-extension path-extension)
