@@ -2,7 +2,7 @@
 
 ;;; File: "_std#.scm"
 
-;;; Copyright (c) 1994-2021 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2025 by Marc Feeley, All Rights Reserved.
 
 ;;;============================================================================
 
@@ -83,6 +83,11 @@
             (macro-fail-check-proper-or-circular-list ,arg-id ,form)))
     ,expr))
 
+;; The finite-list type excludes circular lists.
+
+(define-check-type (list finite-list) #f
+  (lambda (obj) #t)) ;; defer detailed checks to logic traversing the list
+
 ;;; Pair types.
 
 (define-check-type pair 'pair
@@ -341,9 +346,12 @@
     (define prim-vect              (sym "##" name))
     (define prim-vect-length       (sym "##" name '-length))
     (define prim-vect-ref          (sym "##" name '-ref))
+    (define prim-vect-ref-fixnum   (sym "##" name '-ref-fixnum))
     (define prim-vect-set!         (sym "##" name '-set!))
+    (define prim-vect-set!-fixnum  (sym "##" name '-set!-fixnum))
     (define prim-vect-set          (sym "##" name '-set))
     (define prim-vect-set-small    (sym "##" name '-set-small))
+    (define prim-vect-swap!        (sym "##" name '-swap!))
     (define prim-vect->list        (sym "##" name '->list))
     (define prim-list->vect        (sym '##list-> name))
     (define prim-vect-copy         (sym "##" name '-copy))
@@ -362,6 +370,7 @@
     (define prim-subvect-fill!     (sym '##sub name '-fill!))
     (define prim-vect-shrink!      (sym "##" name '-shrink!))
     (define prim-vect-equal?       (sym "##" name '-equal?))
+    (define prim-vect-in-bounds?   (sym "##" name '-in-bounds?))
 
     (define vect?                  (sym name '?))
     (define make-vect              (sym 'make- name))
@@ -369,9 +378,12 @@
     (define vect                   (sym name))
     (define vect-length            (sym name '-length))
     (define vect-ref               (sym name '-ref))
+    (define vect-ref-fixnum        (sym name '-ref-fixnum))
     (define vect-set!              (sym name '-set!))
+    (define vect-set!-fixnum       (sym name '-set!-fixnum))
     (define vect-set               (sym name '-set))
     (define vect-set-small         (sym name '-set-small))
+    (define vect-swap!             (sym name '-swap!))
     (define vect->list             (sym name '->list))
     (define list->vect             (sym 'list-> name))
     (define vect->string           (sym name '->string))
@@ -473,6 +485,7 @@
                                                                            (loop2 (##cdr x)
                                                                                   (##fx+ i 1))))))
                                                                    vect))))))))))))))))))))
+
              `((define-procedure (,vect (elems ,elem-type) ...)
                  (let loop1 ((x elems) (n 0))
                    (if (pair? x)
@@ -506,6 +519,11 @@
                                       (,prim-vect-length ,name))))
                  (,prim-vect-ref ,name k))))
 
+       ,@(if (memq name '(u32vector s32vector u64vector s64vector))
+             `((define-primitive (,vect-ref-fixnum ,name k)
+                 (,prim-vect-ref ,name k)))
+             `())
+
        (define-primitive (,vect-set! ,name k ,elem-name))
 
        ,@(if (memq name '(values))
@@ -519,6 +537,11 @@
                  (,prim-vect-set! ,name k ,elem-name)
                  (void))))
 
+       ,@(if (memq name '(u32vector s32vector u64vector s64vector))
+             `((define-primitive (,vect-set!-fixnum ,name k ,elem-name)
+                 (,prim-vect-set! ,name k ,elem-name)))
+             `())
+
        (define-primitive (,vect-set ,name k val)
          (let ((result (,prim-vect-copy ,name)))
            (,prim-vect-set! result k val)
@@ -537,6 +560,25 @@
                                   (,elem-name ,elem-type))
                  (,prim-vect-set ,name k ,elem-name))))
 
+       (define-primitive (,vect-swap! ,name i j)
+         (let ((temp (,prim-vect-ref ,name i)))
+           (,prim-vect-set! ,name i (,prim-vect-ref ,name j))
+           (,prim-vect-set! ,name j temp)
+           ,name))
+
+       ,@(if (memq name '(values))
+             '()
+             `((define-procedure (,vect-swap!
+                                  (,name ,name)
+                                  (i (index-range
+                                      0
+                                      (,prim-vect-length ,name)))
+                                  (j (index-range
+                                      0
+                                      (,prim-vect-length ,name))))
+                 (,prim-vect-swap! ,name i j)
+                 (void))))
+
        (define-primitive (,vect->list ,name
                                       (start object
                                              0)
@@ -853,8 +895,8 @@ end-of-code
               (case name
                 ((vector)
 #<<end-of-code
-  void *src = ___CAST(void*,&___FIELD(___ARG1,___INT(___ARG2)));
-  void *dst = ___CAST(void*,&___FIELD(___ARG4,___INT(___ARG5)));
+  void *src = ___CAST(void*,&___VECTORELEM(___ARG1,___INT(___ARG2)));
+  void *dst = ___CAST(void*,&___VECTORELEM(___ARG4,___INT(___ARG5)));
   ___SIZE_TS len = ___INT(___FIXSUB(___ARG3,___ARG2)) * ___WS;
 end-of-code
 )
@@ -945,8 +987,8 @@ end-of-code
 )
                 ((values)
 #<<end-of-code
-  void *src = ___CAST(void*,&___FIELD(___ARG1,___INT(___ARG2)));
-  void *dst = ___CAST(void*,&___FIELD(___ARG4,___INT(___ARG5)));
+  void *src = ___CAST(void*,&___FIELD(BOXVALUES,___ARG1,___INT(___ARG2)));
+  void *dst = ___CAST(void*,&___FIELD(BOXVALUES,___ARG4,___INT(___ARG5)));
   ___SIZE_TS len = ___INT(___FIXSUB(___ARG3,___ARG2)) * ___WS;
 end-of-code
 ))
@@ -1086,7 +1128,10 @@ end-of-code
                                          (##declare (generic)) ;; avoid fixnum specific ##eqv?
                                          (,elem= (,prim-vect-ref vect1 i)
                                                  (,prim-vect-ref vect2 i)))
-                                       (loop (##fx- i 1)))))))))))
+                                       (loop (##fx- i 1)))))))))
+
+               (define-prim (,prim-vect-in-bounds? vect k)
+                 (and (##fx<= 0 k) (##fx< k (,prim-vect-length vect))))))
 
        ,@(if define-map-and-for-each
 
@@ -1221,8 +1266,8 @@ end-of-code
                                                  (if (##fx> len-arg max-len)
                                                      (loop (##cdr lst)
                                                            (##cons arg rev-x-y)
+                                                           min-len
                                                            len-arg
-                                                           max-len
                                                            arg-num
                                                            (##fx+ arg-num 1))
                                                      (loop (##cdr lst)
@@ -1319,8 +1364,8 @@ end-of-code
                                                  (if (##fx> len-arg max-len)
                                                      (loop (##cdr lst)
                                                            (##cons arg rev-x-y)
+                                                           min-len
                                                            len-arg
-                                                           max-len
                                                            arg-num
                                                            (##fx+ arg-num 1))
                                                      (loop (##cdr lst)
@@ -1338,6 +1383,7 @@ end-of-code
                                               ,vect-for-each
                                               proc
                                               x-y))))))))))))
+
                )
 
              '())
